unit DebCfgMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, checklst, Registry;

type
  TDebCfgForm = class(TForm)
    LConfigured: TLabel;
    DebSvcList: TCheckListBox;
    AddBtn: TBitBtn;
    DelBtn: TBitBtn;
    ChgBtn: TBitBtn;
    AboutBtn: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure AddBtnClick(Sender: TObject);
    procedure DelBtnClick(Sender: TObject);
    procedure ChgBtnClick(Sender: TObject);
    procedure AboutBtnClick(Sender: TObject);
    procedure DebSvcListClickCheck(Sender: TObject);
  private
    DefaultHelper,
    DefaultTD32: String;
    Function GetDebuggerInfo (Value: String; Var Helper, Debugger: String;
      Var Enabled: Boolean): Boolean;
    Procedure ChangeRegistry (Const Value, Helper, Debugger: String; Enabled: Boolean);
    Procedure RefreshFromRegistry;
  public
    { Public declarations }
  end;

var
  DebCfgForm: TDebCfgForm;

implementation

uses DebCfgProp;

{$R *.DFM}

Const
  ImageFileExecutionOptions =
    'SOFTWARE\Microsoft\Windows NT\CurrentVersion\Image File Execution Options';

Function TDebCfgForm.GetDebuggerInfo (Value: String; Var Helper, Debugger: String; Var Enabled: Boolean): Boolean;
Begin
  Result:= False;
  Value:= UpperCase(Value);
  If Pos('.EXE', Value)=0 Then
    Exit;
  With TRegistry.Create Do
  Try
    RootKey:= HKEY_LOCAL_MACHINE;
    If OpenKey(ImageFileExecutionOptions+'\'+Value, False) Then Begin
      Helper:= ReadString('Debugger');
      Debugger:= ReadString('TD32');
      If Helper<>'' Then
        Enabled:= True
      Else Begin
        Enabled:= False;
        Helper:= ReadString('DisabledDebugger')
      End;
      Result:= (Helper<>'') And (Debugger<>'');
      If DefaultHelper='' Then Begin
        DefaultHelper:= Helper;
        DefaultTD32:= Debugger
      End
    End;
  Finally
    Free
  End
End;

Procedure TDebCfgForm.ChangeRegistry (Const Value, Helper, Debugger: String; Enabled: Boolean);
Begin
  With TRegistry.Create Do
  Try
    RootKey:= HKEY_LOCAL_MACHINE;
    If OpenKey(ImageFileExecutionOptions+'\'+Value, True) Then Begin
      If Enabled Then Begin
        WriteString('Debugger', Helper);
        DeleteValue('DisabledDebugger');
      End Else Begin
        WriteString('DisabledDebugger', Helper);
        DeleteValue('Debugger');
      End;
      WriteString('TD32', Debugger)
    End
  Finally
    Free
  End
End;

Procedure TDebCfgForm.RefreshFromRegistry;
Var
  List: TStrings;
  Helper, Debugger: String;
  Enabled: Boolean;
  i: Integer;
Begin
  DebSvcList.Items.Clear;
  DefaultHelper:= '';
  DefaultTD32:= '';
  With TRegistry.Create Do
  Try
    RootKey:= HKEY_LOCAL_MACHINE;
    If OpenKey(ImageFileExecutionOptions, False) Then Begin
      List:= TStringList.Create;
      Try
        GetKeyNames(List);
        For i:= 0 To List.Count-1 Do
          If GetDebuggerInfo(List[i], Helper, Debugger, Enabled) Then Begin
            DebSvcList.Items.Add(List[i]);
            DebSvcList.Checked[DebSvcList.Items.Count-1]:= Enabled
          End
      Finally
        List.Free
      End
    End
  Finally
    Free
  End
End;

procedure TDebCfgForm.FormCreate(Sender: TObject);
begin
  RefreshFromRegistry
end;


procedure TDebCfgForm.AddBtnClick(Sender: TObject);
begin
  // add a service for debugging
  With DebPropForm Do Begin
    ESvcName.Text:= 'MyService.Exe';
    ESvcDeb.Text:= DefaultHelper;
    EDebugger.Text:= DefaultTD32;
    cbEnabled.Checked:= True;
    If ShowModal=mrOk Then Begin
      ChangeRegistry(ESvcName.Text, ESvcDeb.Text, EDebugger.Text, cbEnabled.Checked);
      RefreshFromRegistry
    End
  End
end;

procedure TDebCfgForm.DelBtnClick(Sender: TObject);
begin
  If DebSvcList.ItemIndex<0 Then
    Exit;
  With TRegistry.Create Do
  Try
    RootKey:= HKEY_LOCAL_MACHINE;
    DeleteKey(ImageFileExecutionOptions+'\'+DebSvcList.Items[DebSvcList.ItemIndex]);
    RefreshFromRegistry
  Finally
    Free
  End
end;

procedure TDebCfgForm.ChgBtnClick(Sender: TObject);
Var
  Helper, Debugger: String;
  DebugEnabled: Boolean;
begin
  If DebSvcList.ItemIndex<0 Then
    Exit;
  If Not GetDebuggerInfo(DebSvcList.Items[DebSvcList.ItemIndex], Helper, Debugger, DebugEnabled) Then
    Exit;
  With DebPropForm Do Begin
    ESvcName.Text:= DebSvcList.Items[DebSvcList.ItemIndex];
    ESvcDeb.Text:= Helper;
    EDebugger.Text:= Debugger;
    cbEnabled.Checked:= DebugEnabled;
    If ShowModal=mrOk Then Begin
      ChangeRegistry(ESvcName.Text, ESvcDeb.Text, EDebugger.Text, cbEnabled.Checked);
      RefreshFromRegistry;
    End
  End
end;

procedure TDebCfgForm.DebSvcListClickCheck(Sender: TObject);
Var
  SvcName, Helper, Debugger: String;
  DebugEnabled: Boolean;
begin
  SvcName:= DebSvcList.Items[DebSvcList.ItemIndex];
  If Not GetDebuggerInfo(SvcName, Helper, Debugger, DebugEnabled) Then
    Exit;
  DebugEnabled:= Not DebugEnabled;
  ChangeRegistry(SvcName, Helper, Debugger, DebugEnabled);
  RefreshFromRegistry
end;

procedure TDebCfgForm.AboutBtnClick(Sender: TObject);
begin
  MessageDlg('This program configures the debug helper SvcDeb.Exe'#13+
             'in order to debug services with TD32 v.5.0.'#13+
             #13+
             'It is part of the NtSvc.Zip Delphi Service components'#13+
             'that may be downloaded from http://www.sawatzki.de'#13+
             #13+
             ' 1997 Peter Sawatzki'#13+
             'EMail: Peter@Sawatzki.de'#13+
             'CSi: 100031,3002',
    mtInformation, [mbOk], 0);
end;

end.
